/*
** TuxServe - XChat plugin.
** (C) 2004 Aypok <aypok@users.sourceforge.net>
**
** TuxServe is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** TuxServe is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with TuxServe; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**
** Last updated 2004-12-22
**
*/



#include <ctype.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "xchat-plugin.h"
#include "config.h"
#include "display.h"
#include "extern.h"
#include "main.h"
#include "menus.h"
#include "queues.h"
#include "sends.h"



extern int alphasort();



/*
** Find the number of the user name passed to it
*/
int find_users_number(char *sender)
{
	int i, user = 0;
	
	for (i = 0; i <= us[0].current_users; i++)
	{
		if (strcmp(us[i].users_name, sender) == 0) { return i; }
	}
	
	return user;
	
//	/* Scroll through all current users */
//	for (i = 0; i <= us[0].current_users; i++)
//	{
//		if (strcmp(us[i].users_name, sender) == 0)
//		{
//			/* Found them */
//			user = i;
//			return user;
//		}
//	}
//	
//	/* If we get here then the user doesn't exist */
//	return 99;
}



/*
** Displays the welcome message to users when they log into the fserve
*/
void display_welcome(xchat_plugin *ph)
{
	char readstring[128], laststring[128];
	FILE *welcome;
	
	/* 1 = random fortune quote - so pipe the output of fortune here.
	** 2 = use "welcome" file with custom welcome message */
	if (fs.show_welcome == 1)
	{
		if ((welcome = popen("fortune", "r")) == NULL) { return; }
		xchat_commandf(ph, "say %iRandom Fortune quote:", fs.colours[1]);
	} else
	{
		sprintf(gs.temp_path, "%swelcome", gs.tuxserve_path);
		if ((welcome = fopen(gs.temp_path, "r")) == NULL) { return; }
	}
	
	/* Read through the file and output all to the screen - unless the
	** current line is the same as the last, or if the line is blank */
	while (fgets(readstring, 128, welcome))
	{
		if ((strcmp(laststring, readstring) != 0) && (strcmp(readstring, "") != 0))
		{
			remove_newline_char(readstring);
			xchat_commandf(ph, "say %i%s", fs.colours[2], readstring);
		}
		strcpy(laststring, readstring);
	}
	xchat_commandf(ph, "say %i---", fs.colours[0]);
	
	/* If we used fortune, close the pipe - else, close the file */
	if (fs.show_welcome == 1) { pclose(welcome); } else { fclose(welcome); }
}



// BROKEN!
/*
** Checks to see if "sender" is still in any of the channels the fserve runs in
*/
int is_user_around(xchat_plugin *ph, char *sender)
{
	int i;
	xchat_list *list;
	xchat_context *context;
//	xchat_commandf(ph, "say moo c2");
	
	list = xchat_list_get(ph, "users");
	xchat_list_next(ph, list);
//	xchat_commandf(ph, "say moo c3");
	
	/* Scroll through all the current serving channels */
	for (i = 0; i <= cs[0].number_of_channels; i++)
	{
		context = xchat_find_context(ph, NULL, cs[i].channel_name);
//		xchat_commandf(ph, "say moo c4.%i", i);
		
		/* Scroll through all users in that channel to see if the
		** user we're looking for is in there */
		while(xchat_list_next(ph, list))
		{
			/* Is this person is still in the channel? */
			if ((strcmp(xchat_list_str(ph, list, "nick"), sender)) == 0)
			{
				/* Yep, they are - return 1 */
//				xchat_commandf(ph, "say moo c5");
				xchat_list_free(ph, list);
				return 1;
			} else
			{
//				xchat_commandf(ph, "msg Aypok \"%s\" vs \"%s\"", xchat_list_str(ph, list, "nick"), sender);
			}
		}
	}
	/* They're not around anymore - return 0 */
//	xchat_commandf(ph, "say moo c6");
	xchat_list_free(ph, list);
	
	return 0;
}



/*
** Sets the context to output PRIVMSGs to the right window
*/
int set_context(xchat_plugin *ph, char *set_to)
{
	xchat_context *context;
	context = xchat_find_context(ph, NULL, set_to);
	if (xchat_set_context(ph, context))
	{
		return 1;
	} else
	{
		return 0;
	}
}



/*
** Displays the info file for that dir to the user
*/
void display_dir_info(xchat_plugin *ph, char *info_path, char *local_path)
{
	char read_string[256];//, *string = NULL;
	FILE *info_file;
	
	/* Open the info file for reading */
	if((info_file = fopen(info_path, "r")) != NULL)
	{
		/* Header type thing */
		xchat_commandf(ph, "say %i---", fs.colours[0]);
		xchat_commandf(ph, "say %iInformation about \"%i%s%i\":", fs.colours[2], fs.colours[3], local_path, fs.colours[2]);
		xchat_commandf(ph, "say ");
		
		/* Read through and print out everything in the file */
		while (fgets(read_string, 256, info_file))
		{
			remove_newline_char(read_string);
			xchat_commandf(ph, "say %i%s", fs.colours[1], read_string);
		}
		
		xchat_commandf(ph, "say %i---", fs.colours[0]);
		xchat_commandf(ph, "say ");
		fclose(info_file);
	}
}



/*
** Finds, sorts, and lists all files/dirs in the current dir - then
** sends the list to the user
*/
void list_current_dir(xchat_plugin *ph, char *sender)
{
	int count, i, place[2], file_number = 1, dir_number = 1, user = 0;
	unsigned long int size;
	char first_char[6], current_file[512], local_path[256], output[256];
	char size_string[3], path_copy[512], final_path[512];
	struct direct **files;
	struct stat buf;
		
	/* Find the number of this user */
	for (i = 0; i <= us[0].current_users; i++) { user = find_users_number(sender); }
	
	/* Send all output to the DCC window that is open with this person */
	set_context(ph, us[user].users_name);
	
	if (chdir(us[user].working_dir[0])) { xchat_commandf(ph, "say %iError changing to \"%s\".", fs.colours[4], us[user].working_dir[0]); }
	
	/* This bit removes triggerdir[x] from the path it shows - so the user
	** doesn't see the full system path */
	i = us[user].users[0];
	place[0] = (strlen(ts.triggerdir[i]) - 1);
	place[1] = strlen(us[user].working_dir[0]);
	for (i = 0; i <= place[1]; i++) { local_path[i] = us[user].working_dir[0][(place[0] + i)]; }
	
	/* Find and print the info for each dir - if any exist */
	if (fs.show_dir_info)
	{
		snprintf(path_copy, 512, "%s", us[user].working_dir[0]);
		get_last_dir(path_copy);
		snprintf(final_path, 512, "%s.TuxServe_info-%s", us[user].working_dir[0], path_copy);
		display_dir_info(ph, final_path, local_path);
	}
	
	xchat_commandf(ph, "say %i[%i%s%i]", fs.colours[0], fs.colours[3], local_path, fs.colours[0]);
	count = scandir(us[user].working_dir[0], &files, NULL, alphasort);
	
	/* If no files found, make a non-selectable menu item */
	if (count <= 0) { xchat_commandf(ph, "say %iThis directory is empty.", fs.colours[4]); }
//	printf("Number of files = %d\n", count);
	
	/* Find if it is a file or dir - print dirs first */
	for (i = 1; i < (count + 1); ++i)
	{
		sprintf(current_file, "%s%s", us[user].working_dir[0], files[i-1]->d_name);
//		if (stat(current_file, &buf) != 0)
		if (stat(current_file, &buf))
		{
			xchat_commandf(ph, "say %iERROR:: You do not have permission to view this dir.", fs.colours[4]);
			xchat_commandf(ph, "say %iPlease let whomever is running this fserve know of this error, and that they should visit %ihttp://tuxserve.sf.net/faq.php#nopermission%i for information on how to fix this problem. Thanks.", fs.colours[4], fs.colours[5], fs.colours[4]);
		} else
		{
			sprintf(first_char, "%o", buf.st_mode);
			
			/* Is it a directory? */
			if (first_char[0] == '4')
			{
				/* Skip the "." dir */
				if (strcmp(files[i-1]->d_name, ".") == 0) { continue; }
				
				/* If the user wants to hide hidden dirs */
				if (!fs.show_hidden)
				{
					/* Check if it's a hidden dir (but not ".." - eg, go-up-a-dir) */
					if ((files[i-1]->d_name[0] == '.') && (strcmp(files[i-1]->d_name, "..") != 0)) { continue; }
				}
				
				/* Display it */
				sprintf(output, "%-2.0i%id:%i %s/", fs.colours[0], dir_number, fs.colours[3], files[i-1]->d_name);
				if (dir_number < 10)
				{
					xchat_commandf(ph, "say %s", output);
				} else
				{
					xchat_commandf(ph, "say %s", output);
				}
				dir_number++;
			}
		}
	}
	
	/* Go through the list of items in this dir, and print just the files */
	for (i = 1; i < (count + 1); ++i)
	{
		sprintf(current_file, "%s%s", us[user].working_dir[0], files[i-1]->d_name);
//		if (stat(current_file, &buf) != 0)
		if (stat(current_file, &buf))
		{
			xchat_commandf(ph, "say %iERROR:: You do not have permission to view this file.", fs.colours[4]);
			xchat_commandf(ph, "say %iPlease let whomever is running this fserve know of this error, and that they should visit %ihttp://tuxserve.sf.net/faq.php#nopermission%i for information on how to fix this problem. Thanks.", fs.colours[4], fs.colours[5], fs.colours[4]);
		} else
		{
			sprintf(first_char, "%o", buf.st_mode);
			
			/* Is it a file? */
			if (first_char[0] == '1')
			{
				/* If the user wants to hide hidden files */
				if (!fs.show_hidden)
				{
					/* Check if it's a hidden file */
					if (files[i-1]->d_name[0] == '.') { continue; }
				}
				
				/* For displaying the size in bytes, kb, mb, gb, etc */
				size = buf.st_size;
				
				/* If the files size is less than 1kb */
				if (size < 1024) { sprintf(size_string, "b"); }
				
				/* If the file is between 1kb and 1mb */
				if ((size < 1048576) && (size > 1023)) { size = (size / 1024); sprintf(size_string, "kb"); }
				
				/* If the file is between 1mb and 1gb */
				if ((size < 1073741824) && (size > 1048575)) { size = (size / 1048576); sprintf(size_string, "mb"); }
				
				/* If it's over 1gb */
				if (size > 1073741823) { size = (size / 1073741824); sprintf(size_string, "gb"); }
				
				/* Display it */
				sprintf(output, "%-2.0i%if:%i %s  %i", fs.colours[3], file_number, fs.colours[4], files[i-1]->d_name, fs.colours[2]);
				xchat_commandf(ph, "say %s %u%s", output, size, size_string);
	
				file_number++;
			}
		}
	}
	
	xchat_commandf(ph, "say %i[%iEnd of list%i]", fs.colours[0], fs.colours[3], fs.colours[0]);
}



/*
** Changes the current working directory using tags
*/
int change_dir(xchat_plugin *ph, char *dir_ident, char *sender)
{
	int count, i, j, dir_number, dir_counter = 1, user = 0;
	char first_char[6], dir_code[9], current_dir_name[512], *some_string = NULL;
	char temp_string[256];
	struct direct **files;
	struct stat buf;
	
	/* Find the number of this user */
	for (i = 0; i <= us[0].current_users; i++)
	user = find_users_number(sender);
	set_context(ph, us[user].users_name);

	/* Remove the colon and the "d" - then convert to an int */
	for (i = 0; i < 8; i++)
	{
		dir_code[i] = dir_ident[i];
		if (dir_code[i] == 'd') { dir_code[i] = '\0'; }
//		if (!isdigit(dir_code[i])) { dir_code[i] = '\0'; }
	}
	dir_number = atoi(dir_code);

	/* Set the start number of dirs. If no hidden files are shown, it starts at
	** 1, else it starts at 2 - to skip the "." dir */
	if (fs.show_hidden)
	{
		j = 2;
	} else
	{
		j = 1;
	}
	
	/* Find the dir that this code links to */
	count = scandir(us[user].working_dir[0], &files, NULL, alphasort);
	for (i = j; i < (count + 1); ++i)
	{
		/* If it's a hidden dir and the user has set it to not show hidden files,
		** then skip it - so it's as if this dir was not counted */
		if ((files[i-1]->d_name[0] == '.') && (strcmp(files[i-1]->d_name, "..") != 0))
		{	
			if (!fs.show_hidden) { continue; }
		}
		
//		xchat_commandf(ph, "say file=\"%i\"");
		/* We can't add a dir seperator on the end here, as if it was a file
		** it'd cause an error - so, we add a "/" when we know it's a file*/
		sprintf(current_dir_name, "%s%s", us[user].working_dir[0], files[i-1]->d_name);
		if (stat(current_dir_name, &buf))
		{
			/* Error */
			xchat_commandf(ph, "say %iERROR:: Cannot change directory.", fs.colours[0]);
			return 0;
		}
		sprintf(first_char, "%o", buf.st_mode);
		
		if (first_char[0] == '4')
		{
			/* If it is a dir, add the dir seperator at the end */
			sprintf(current_dir_name, "%s/", current_dir_name);
			
//			xchat_commandf(ph, "say dir_counter=%i ][ dir_number=%i", dir_counter, dir_number);
			if (dir_counter == dir_number)
			{
//				xchat_commandf(ph, "say dir=\"%s\" - td=\"%s\" - wd=\"%s\"", files[i-1]->d_name, ts.triggerdir[(us[user].users[0])], us[user].working_dir[0]);
				/* If the user wants to go up a dir, make sure we're not in the
				** root of the trigger_dir */
				if (strcmp(files[i-1]->d_name, "..") == 0)
				{
					strcpy(temp_string, us[user].working_dir[0]);
					some_string = remove_last_dir(temp_string);
					
					/* Root of trigger_dir */
					if (strcmp(ts.triggerdir[(us[user].users[0])], us[user].working_dir[0]) == 0)
					{
						/* Let them know they can't go up any higher */
						xchat_commandf(ph, "say %iERROR:: You cannot go up a directory from here - this is the root of this trigger.", fs.colours[0]);
						return 0;
					}
					
					/* Remove old dir name first */
//					some_string = remove_last_dir(us[user].working_dir[0]);
//					xchat_commandf(ph, "say dir=\"%s\"", some_string);
					sprintf(current_dir_name, "%s/", some_string);
				}
				sprintf(us[user].working_dir[0], "%s", current_dir_name);
				
//				xchat_commandf(ph, "say %s/", current_dir_name);
				/* When this function returns, the function list_current_dir() is
				** called. That changes the actual dir and lists the files in it */
				return 1;
			} else
			{
				dir_counter++;
			}
		}
	}
//	xchat_commandf(ph, "say moo00 - should never get here :)");
	
	return 0;
}



/*
** Changes the current working directory using the "cd <dir name>" command
*/
int change_dir_cd(xchat_plugin *ph, char *dir_name, char *sender)
{
	int i, user = 0;
	struct stat buf;
	char first_char[6], current_dir_name[512], temp_string[512], *some_string = NULL;
	
	/* Find the number of this user */
	for (i = 0; i <= us[0].current_users; i++)
	user = find_users_number(sender);
	set_context(ph, us[user].users_name);
	
	/* If it's a hidden dir and the user has set it to not show hidden files,
	** then give the "not found" error and return */
	if ((dir_name[0] == '.') && ((strcmp(dir_name, "..") != 0) && (strcmp(dir_name, "../") != 0)))
	{
		if (!fs.show_hidden)
		{
			xchat_commandf(ph, "say %iERROR:: Can't change directory.", fs.colours[0]);
			xchat_commandf(ph, "say %iThis fserve is case-sensitive - make sure you use the correct case. Type \"help case\" for more information on this.");
			return 0;
		}
	}
	
	/* Sort out some strings to get the dir info */
	some_string = get_first_dir(dir_name);
//	some_string = remove_last_dir(us[user].working_dir[0]);
//	sprintf(current_dir_name, "%s/%s/", some_string, dir_name);
	sprintf(current_dir_name, "%s%s/", us[user].working_dir[0], some_string);
//	xchat_commandf(ph, "say cdn=\"%s\" DID I FIX IT YET!?!?!?", current_dir_name);
	if (stat(current_dir_name, &buf))
	{
		/* Dir problem - doesn't exist, no permission, etc */
		xchat_commandf(ph, "say %iERROR:: Can't change directory.", fs.colours[0]);
		xchat_commandf(ph, "say %iThis fserve is case-sensitive - make sure you use the correct case. Type \"help case\" for more information on this.");
		return 0;
	}
	sprintf(first_char, "%o", buf.st_mode);
	
	/* If it is a dir */
	if (first_char[0] == '4')
	{
		/* Do they want to go up a level? */
		if (strcmp(dir_name, "..") == 0)
		{
			strcpy(temp_string, us[user].working_dir[0]);
			some_string = remove_last_dir(temp_string);
//			xchat_commandf(ph, "say ss=\"%s\" - td=\"%s\" - wd=\"%s\"", some_string, ts.triggerdir[(us[user].users[0])], us[user].working_dir[0]);
			
			/* Check that they're not in the root of the trigger_dir */
			if (strcmp(ts.triggerdir[(us[user].users[0])], us[user].working_dir[0]) == 0)
			{
				/* Let them know they can't go up any higher */
				xchat_commandf(ph, "say %iERROR:: You cannot go up a directory from here - this is the root of this trigger.", fs.colours[0]);
				return 0;
			}
			sprintf(current_dir_name, "%s/", some_string);
			
			/* Remove old dir name first */
//			some_string = remove_last_dir(us[user].working_dir[0]);
////			xchat_commandf(ph, "say dir=\"%s\"", some_string);
//			sprintf(current_dir_name, "%s", some_string);
		}
		
		sprintf(us[user].working_dir[0], "%s", current_dir_name);
		
		return 1;
	} else
	{
		/* It's not a dir */
		xchat_commandf(ph, "say %iERROR:: \"%s\" is not a directory.", fs.colours[0], dir_name);
		return 0;
	}
}



/*
** Adds the file requested to the queue using the tags
*/
void requested_file(xchat_plugin *ph, char *file_ident, char *sender)
{
	int count, i, j, file_number, file_counter = 1, user;
	char first_char[6], file_code[9], current_file_name[512];
	char place[2], local_path[256];
	struct direct **files;
	struct stat buf;
	
	/* Find the number of this user */
	user = find_users_number(sender);
	set_context(ph, us[user].users_name);
	
	/* Remove the colon and the "f" - then convert to an int */
	for (i = 0; i < 8; i++)
	{
		file_code[i] = file_ident[i];
		if (file_code[i] == 'f') { file_code[i] = '\0'; }
	}
	file_number = atoi(file_code);
	
	/* Set the start number of dirs. If no hidden files are shown, it starts at
	** 1, else it starts at 2 - to skip the "." dir */
	if (fs.show_hidden)
	{
		j = 2;
	} else
	{
		j = 1;
	}
	
	/* Find the file that this code links to */
	count = scandir(us[user].working_dir[0], &files, NULL, alphasort);
	for (i = j; i < (count + 1); ++i)
	{		
		sprintf(current_file_name, "%s%s", us[user].working_dir[0], files[i-1]->d_name);
		if (stat(current_file_name, &buf)) { xchat_commandf(ph, "say %iERROR:: Can't find file.", fs.colours[0]); }
		sprintf(first_char, "%o", buf.st_mode);
		
		if (first_char[0] == '1')
		{
			/* If it's a hidden file and the user has set it to not show hidden
			** files, then skip it - so it's as if this dir was not counted */
			if (files[i-1]->d_name[0] == '.') { if (!fs.show_hidden) { continue; } }
			
			if (file_counter == file_number)
			{	
				/* Check if send_now is on - if so, does the file qualify? Make
				** sure it's not paused, too */
//				xchat_commandf(ph, "say fs.sn=\"%i\" fs.sn*1k=\"%i\" buf.sts=\"%i\" fs.p=\"%i\"", fs.send_now, (fs.send_now * 1024), buf.st_size, fs.paused);
				if (((fs.send_now) && ((fs.send_now * 1024) >= buf.st_size)) && (!fs.paused))
				{
					place[0] = (strlen(ts.triggerdir[0]) - 1);
					place[1] = strlen(current_file_name);
					for (j = 0; j <= place[1]; j++)
					{
						local_path[j] = current_file_name[(place[0] + j)];
					}
					
					send_file(ph, current_file_name, sender, local_path);
					return;
				}
				
				/* Check that they have free queue slots */
				if (fs.queue < fs.max_queues)
				{
					if (query_free_queues(ph, sender) < fs.user_queues)
					{
						/* If the fserve is not paused, check free sends (if
						** there are free slots, it will start sends) */
						if (!fs.paused)
						{
							/* Free sends? */
							if (check_sends(ph, current_file_name, sender) == 1)
							{
								/* Check if priority queueing is on */
								if (fs.priority_queue) { /* xchat_commandf(ph, "say add_prefix"); */ add_mode_prefix_user(&user, sender); }
								
								/* Nope - add the file to the queue instead */
								does_queue_exist(ph, sender, current_file_name);
								return;
							}
						} else
						{
							/* Check if priority queueing is on */
							if (fs.priority_queue) { /* xchat_commandf(ph, "say add_prefix"); */ add_mode_prefix_user(&user, sender); }
							
							/* Fserve is paused - add file to the queue */
							does_queue_exist(ph, sender, current_file_name);
							return;
						}
					} else
					{
						xchat_commandf(ph, "say %iSorry, you've filled all your queue slots.", fs.colours[4]);
					}
				} else
				{
					xchat_commandf(ph, "say %iSorry, all the queue slots have been filled. Try again later.", fs.colours[4]);
				}
				
				return;
			} else
			{
				file_counter++;
			}
		}
	}
	
	return;
}



/*
** Adds the file requested to the queue using the "get <file name>" command
*/
void requested_file_get(xchat_plugin *ph, char *file_name, char *sender)
{
	int user;
	char this_file[256], first_char[6];
	struct stat buf;
		
	/* Find the number of this user */
	user = find_users_number(sender);
	set_context(ph, us[user].users_name);
	
	/* If it's a hidden file and the user has set it to not show hidden files,
	** then give the "not found" error and return */
	if (file_name[0] == '.')
	{
		if (!fs.show_hidden)
		{
			xchat_commandf(ph, "say %iERROR:: Can't find file.", fs.colours[0]);
			xchat_commandf(ph, "say %iThis fserve is case-sensitive - make sure you use the correct case. Type \"help case\" for more information on this.");
			return;
		}
	}
	
	/* Check that the file exists */
	sprintf(this_file, "%s%s", us[user].working_dir[0], file_name);
	if (stat(this_file, &buf))
	{
		xchat_commandf(ph, "say %iERROR:: Can't find file.", fs.colours[0]);
		xchat_commandf(ph, "say %iThis fserve is case-sensitive - make sure you use the correct case. Type \"help case\" for more information on this.");
		return;
	}
	
	/* Check it's not a directory */
	sprintf(first_char, "%o", buf.st_mode);
	if (first_char[0] == '4')
	{
		xchat_commandf(ph, "say %iThat is a directory, not a file.", fs.colours[0]);
		return;
	} else
	{
		/* Check if send_now is on - if so, does the file qualify? Make sure
		** TuxServe isn't paused, too */
		if (((fs.send_now) && ((fs.send_now * 1024) >= buf.st_size)) && (!fs.paused))
		{
			send_file(ph, this_file, sender, file_name);
		}
		
		/* Check that they have free queue slots */
		if (fs.queue < fs.max_queues)
		{
			if (query_free_queues(ph, sender) < fs.user_queues)
			{
				/* If the fserve is not paused, check free sends (if
				** there are free slots, it will start sends) */
				if (!fs.paused)
				{
					/* Free sends? */
					if (check_sends(ph, this_file, sender) == 1)
					{
						/* Check if priority queueing is on */
						if (fs.priority_queue) { /* xchat_commandf(ph, "say add_prefix"); */ add_mode_prefix_user(&user, sender); }
						
						/* Nope - add the file to the queue instead */
						does_queue_exist(ph, sender, this_file);
						return;
					}
				} else
				{
					/* Check if priority queueing is on */
					if (fs.priority_queue) { /* xchat_commandf(ph, "say add_prefix"); */ add_mode_prefix_user(&user, sender); }
					
					/* Fserve is paused - add file to the queue */
					does_queue_exist(ph, sender, this_file);
					return;
				}
			} else
			{
				xchat_commandf(ph, "say %iSorry, you've filled all your queue slots.", fs.colours[4]);
			}
		} else
		{
			xchat_commandf(ph, "say %iSorry, all the queue slots have been filled. Try again later.", fs.colours[4]);
		}
	}	
}



/*
** A user has left the fserve.
*/
void user_left_fserve(xchat_plugin *ph, char *sender)
{
	int i, user;
	char message[256];
	
	/* Check we haven't already removed this user */
	if (strcmp(us[0].last_user, sender) == 0) { return; }
	strcpy(us[0].last_user, sender);
	
	/* Find the number of this user */
	user = find_users_number(sender);
	
	/* If they queued any files, let them know about the "my_queues" trigger */
	if (us[user].queued_flag)
	{
		sprintf(message, "%iIf you wish to check on your queues, please do not log back into the fserve. Instead, use the command \"%i/ctcp %s my_queues%i\". That will send you a list of all your queues, and what position they are in, via a notice.", fs.colours[4], fs.colours[0], xchat_get_info(ph, "nick"), fs.colours[4]);
		send_notice(ph, sender, message);
	}
	
	/* Sort out the list of queues */
	for (i = user; i < us[0].current_users; i++)
	{
		strcpy(us[i].sender_, us[(i + 1)].sender_);
		strcpy(us[i].working_dir[0], us[(i + 1)].working_dir[0]);
		strcpy(us[i].users_name, us[(i + 1)].users_name);
//		xchat_commandf(ph, "say us[%i].users_name=\"%s\" == us[%i].users_name=\"%s\"", i, us[i].users_name, (i + 1), us[(i + 1)].users_name);
		
		us[i].prefix = us[(i + 1)].prefix;
		us[i].users[0] = us[(i + 1)].users[0];
//		us[i].users[1] = us[(i + 1)].users[1];
//		us[i].users[2] = us[(i + 1)].users[2];
		us[i].offered_dcc_chat = us[(i + 1)].offered_dcc_chat;
		
		us[i].tag_flag = us[(i + 1)].tag_flag;
		us[i].queued_flag = us[(i + 1)].queued_flag;
		us[i].trigger_used = us[(i + 1)].trigger_used;
	}
	
	if (us[0].current_users) { us[0].current_users--; }
	
	/* If there are no users, clear the users list to make sure nothing is left
	** over from previous users to screw things up */
	if (us[0].current_users == 0)
	{
//		xchat_commandf(ph, "say CLEAR ALL!");
		for (i = 0; i <= MAXUSERS; i++)
		{
			strcpy(us[i].users_name, "");
			strcpy(us[i].sender_, "");
			strcpy(us[i].working_dir[0], "");
			strcpy(us[0].last_user, "");
			us[i].prefix = '\0';
			us[i].tag_flag = 0;
			us[i].queued_flag = 0;
			us[i].users[0] = 0;
			us[i].offered_dcc_chat = 0;
			us[i].trigger_used = 0;
			us[0].current_users = 0;
		}
	}
	
	if (!set_context(ph, sender)) { return; }
	if (fs.auto_close) { xchat_command(ph, "close"); }
}



/*
** Handles "quit" and "exit" commands when sent from the user to the fserve
*/
void exit_fserve(xchat_plugin *ph, char *sender)
{
	if (!set_context(ph, sender)) { return; }
	xchat_commandf(ph, "say %iToodle pip.", fs.colours[4]);
	xchat_commandf(ph, "dcc close chat %s", sender);
	user_left_fserve(ph, sender);
}



/*
** Responds to "stats", when typed in the DCC fserve chat session, with a list
** of all the stats about this fserve
*/
void show_stats(xchat_plugin *ph, char *sender)
{
	set_context(ph, sender);
	
	xchat_commandf(ph, "say %i---", fs.colours[0]);
	xchat_commandf(ph, "say %iStatistics:", fs.colours[1]);
	xchat_commandf(ph, "say %iRecord send speed:%i %i", fs.colours[2], fs.colours[4], fs.record_speed);
	xchat_commandf(ph, "say %iRecord made by:%i %s", fs.colours[2], fs.colours[4], fs.record_user);
	xchat_commandf(ph, "say %iTotal files sent:%i %i", fs.colours[2], fs.colours[4], fs.files_sent);
	xchat_commandf(ph, "say %iTotal data sent:%i %i%s", fs.colours[2], fs.colours[4], fs.data_sent, fs.data_type);
	xchat_commandf(ph, "say %iAccessed:%i %i", fs.colours[2], fs.colours[4], fs.accessed);
	xchat_commandf(ph, "say %iFailed sends:%i %i", fs.colours[2], fs.colours[4], fs.sends_failed);
	xchat_commandf(ph, "say ");
	xchat_commandf(ph, "say %iXChat version:%i %s", fs.colours[2], fs.colours[4], xchat_get_info(ph, "version"));
	xchat_commandf(ph, "say %iTuxServe version:%i %s", fs.colours[2], fs.colours[4], gs.version);
	xchat_commandf(ph, "say %i---", fs.colours[0]);
	
	return;
}



/*
** Responds to "help", when typed in the DCC fserve chat session, with a list
** of all the help for this fserve
*/
void show_help(xchat_plugin *ph, char *sender, char *help_type)
{
	set_context(ph, sender);
	
	/* Header */
	xchat_commandf(ph, "say %i---", fs.colours[0]);
	xchat_commandf(ph, "say %iHelp:", fs.colours[1]);
	xchat_commandf(ph, "say %i ", fs.colours[1]);
	
	/* Help on using the tags system */
	if (strcasecmp(help_type, "tags") == 0)
	{
		xchat_commandf(ph, "say %iThe tags are a very simple way of navigating the fserve's file system and requesting files.", fs.colours[1]);
		xchat_commandf(ph, "say %iTo the left of every file is a tag in a bold and %icoloured%i font - it's the several digit code starting with a number and ending in the letter \"f\".", fs.colours[1], fs.colours[3], fs.colours[1]);
		xchat_commandf(ph, "say %iIf you wish to get a file, just enter the file's tag. For example, if you see a file like this:", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %i8f:%i tuxserve-0.0.8alpha.tar.gz%i   44kb", fs.colours[3], fs.colours[4], fs.colours[2]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iInstead of typing \"get tuxserve-0.0.8alpha.tar.gz\", you can simply type \"8f\" - it will do exactly the same thing.", fs.colours[1]);
		xchat_commandf(ph, "say %iThe same goes for directories - except their tags are %ithis colour%i and end in the letter \"d\".", fs.colours[1], fs.colours[0], fs.colours[1]);
		xchat_commandf(ph, "say %iIf, for example, you wanted to enter this directory:", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %i3d:%i source code/", fs.colours[0], fs.colours[3]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iYou can just type \"3d\" instead of \"cd source code\". If you prefer to not use the tags, the normal \"get\" and \"cd\" commands are there for you to use.", fs.colours[1]);
		xchat_commandf(ph, "say %iAlso, when you use the tags to change directory, the contents of the new directory are automatically listed for you, so you don't need to type \"dir\" or \"ls\".", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iTo go back a directory (up a level), use the \"../\" directory, which is usually \"2d\" - but it varies from system to system.", fs.colours[1]);
	}
	
	/* Displays TuxServe's license - the GNU/GPL */
	if (strcasecmp(help_type, "license") == 0)
	{
		xchat_commandf(ph, "say %iWhat follows is the license which TuxServe is released under.", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iTuxServe (C) 2004 Aypok <aypok@users.sourceforge.net>", fs.colours[1]);
		xchat_commandf(ph, "say %ihttp://tuxserve.sf.net/", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iTuxServe is free software; you can redistribute it and/or modify", fs.colours[1]);
		xchat_commandf(ph, "say %iit under the terms of the GNU General Public License as published by", fs.colours[1]);
		xchat_commandf(ph, "say %ithe Free Software Foundation; either version 2 of the License, or", fs.colours[1]);
		xchat_commandf(ph, "say %i(at your option) any later version.", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iTuxServe is distributed in the hope that it will be useful,", fs.colours[1]);
		xchat_commandf(ph, "say %ibut WITHOUT ANY WARRANTY; without even the implied warranty of", fs.colours[1]);
		xchat_commandf(ph, "say %iMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the", fs.colours[1]);
		xchat_commandf(ph, "say %iGNU General Public License for more details.", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iYou should have received a copy of the GNU General Public License", fs.colours[1]);
		xchat_commandf(ph, "say %ialong with TuxServe; if not, write to the Free Software", fs.colours[1]);
		xchat_commandf(ph, "say %iFoundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", fs.colours[1]);
	}
	
	/* Generic help on case sensitivity */
	if (strcasecmp(help_type, "case") == 0)
	{
		xchat_commandf(ph, "say %iThis fserve is running on a *nix based operating system, which means that it is case sensitive.", fs.colours[1]);
		xchat_commandf(ph, "say %i\"case senstitive\" means that the computer differenciates between upper case (eg, \"HELLO\") and lower case (eg, \"hello\").", fs.colours[1]);
		xchat_commandf(ph, "say %iTo a case sensitive operating system like this one, the 2 above examples are NOT the same word!", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iPlease be careful when using the \"get\" and \"cd\" commands, as if you use the wrong case the system will not know what you mean and give an error.", fs.colours[1]);
		xchat_commandf(ph, "say %iThe best way to avoid this sort of problem is to use the tags system - type \"help tags\" for more information.", fs.colours[1]);
		xchat_commandf(ph, "say %iAlternatively, you could copy and paste the names of the files/dirs, instead of typing them.", fs.colours[1]);
	}
	
	/* Generic help and topic list */
	if (strcasecmp(help_type, "-") == 0)
	{
		xchat_commandf(ph, "say %iYou wish... I mean... Uh, no generic help available yet.", fs.colours[1]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iHelp topics:", fs.colours[1]);
		xchat_commandf(ph, "say %i -> \"%ihelp tags%i\"    :: %iShows help on using the tags system.", fs.colours[1], fs.colours[2], fs.colours[1], fs.colours[4]);
		xchat_commandf(ph, "say %i -> \"%ihelp license%i\" :: %iDisplays the license TuxServe is released under.", fs.colours[1], fs.colours[2], fs.colours[1], fs.colours[4]);
		xchat_commandf(ph, "say %i -> \"%ihelp case%i\"    :: %iGives help regarding case sensitivity.", fs.colours[1], fs.colours[2], fs.colours[1], fs.colours[4]);
		xchat_commandf(ph, "say %i ", fs.colours[1]);
		xchat_commandf(ph, "say %iTuxServe: an fserve for XChat - http://tuxserve.sf.net/", fs.colours[1]);
	}
	
	/* Footer */
	xchat_commandf(ph, "say %i---", fs.colours[0]);
	
	return;
}



/*
** Responds to "who", when typed in the DCC fserve chat session, with a list
** of all the users who are currently browsing this fserve
*/
void show_who(xchat_plugin *ph, char *sender)
{
	int i;
	
	if (strcmp(sender, "@ts_internal@") != 0)
	{
		set_context(ph, sender);
		xchat_commandf(ph, "say %i---", fs.colours[0]);
		xchat_commandf(ph, "say %iCurrent users:%i %i/%i", fs.colours[2], fs.colours[4], us[0].current_users, fs.max_users);
		
		/* This loop runs through all current users and displays their info */
		for (i = 0; i < us[0].current_users; i++)
		{	
			xchat_commandf(ph, "say %iUser%i %i %iis%i %s", fs.colours[2], fs.colours[4], (i + 1), fs.colours[1], fs.colours[4], us[i].users_name);
		}
		xchat_commandf(ph, "say %i---", fs.colours[0]);
		
		return;
	} else
	{
		xchat_printf(ph, "%i---", fs.colours[0]);
		xchat_printf(ph, "%iCurrent users:%i %i/%i", fs.colours[2], fs.colours[4], us[0].current_users, fs.max_users);
		
		/* This loop runs through all current users and displays their info */
		for (i = 0; i < us[0].current_users; i++)
		{	
			xchat_printf(ph, "%iUser%i %i %iis%i %s", fs.colours[2], fs.colours[4], (i + 1), fs.colours[1], fs.colours[4], us[i].users_name);
		}
		xchat_printf(ph, "%i---", fs.colours[0]);
		
		return;
	}
	
	return;
}



/*
** Updates the stats and then saves them to the config file
*/
void update_stats(xchat_plugin *ph, char *filename, char *sender, unsigned long int speed)
{
	xchat_list *list;
	
	/* If that file went faster than any other, set it as the new record speed
	** and set the record speeds user to whomever it was sent to.
	** It's divided by 1024 as the speed given is in bytes/sec, and we
	** want to use Kilobytes/sec - makes it easier for us :)*/
//	xchat_commandf(ph, "say moo us 1");
//	xchat_commandf(ph, "say moo s=\"%u\" ts=\"%i\" ... s2=\"%u\"", speed, fs.record_speed, (speed / 1024));
	
	if ((speed / 1024) > fs.record_speed)
	{
		fs.record_speed = (speed / 1024);
		strcpy(fs.record_user, sender);
	}
//	xchat_commandf(ph, "say moo us 2");
	
	/* Add 1 to the number of files sent */
	fs.files_sent++;
//	xchat_commandf(ph, "say moo us 3");
	
	/* Find the size of the file, then add the size (in bytes) to the total
	** number of bytes sent from this fserve */
	list = xchat_list_get(ph, "dcc");
//	xchat_commandf(ph, "say moo us 4");
	
	while(xchat_list_next(ph, list))
	{
//		xchat_commandf(ph, "say moo us 5");
		if ((strcmp(xchat_list_str(ph, list, "nick"), sender)) == 0)
		{
//			xchat_commandf(ph, "say moo us 6");
//			if ((strcmp(xchat_list_str(ph, list, "file"), filename)) == 0)
//			{
				if ((xchat_list_int(ph, list, "status") == 3) && (xchat_list_int(ph, list, "type") == 0))
				{
//					xchat_commandf(ph, "say moo us 7");
					
					/* Turn the size into the right type - eg, kb, mb, gb, etc.
					** If we add it with the wrong size, the data will be wrong */
					if (strcmp(fs.data_type, "b") == 0)
					{
						/* Leave it in bytes */
						fs.data_sent = (fs.data_sent + xchat_list_int(ph, list, "size"));
					}
					
					/* Convert the new data to kb before adding it */
					if (strcmp(fs.data_type, "kb") == 0)
					{
						fs.data_sent = (fs.data_sent + (xchat_list_int(ph, list, "size") / 1024));
					}
					
					/* Convert the new data to kb before adding it */
					if (strcmp(fs.data_type, "mb") == 0)
					{
						fs.data_sent = (fs.data_sent + (xchat_list_int(ph, list, "size") / 1048576));
					}
					
					/* Convert the new data to kb before adding it */
					if (strcmp(fs.data_type, "gb") == 0)
					{
						fs.data_sent = (fs.data_sent + (xchat_list_int(ph, list, "size") / 1073741824));
					}
					
					/* Now make sure it never goes above 1023. If it does, then
					** move upto the next data_type (except for gb) */
					if ((fs.data_sent > 1023) && (strcmp(fs.data_type, "gb") != 0))
					{
						fs.data_sent = (fs.data_sent / 1024);
						
						/* Change the data_type accordingly */
						if (strcmp(fs.data_type, "mb") == 0) { sprintf(fs.data_type, "gb"); }
						if (strcmp(fs.data_type, "kb") == 0) { sprintf(fs.data_type, "mb"); }
						if (strcmp(fs.data_type, "b") == 0) { sprintf(fs.data_type, "kb"); }
					}
			
					break;
				}
//			}
		}
	}
//	xchat_commandf(ph, "say moo us 8");
	xchat_list_free(ph, list);
	
	/* Save the stats to the config file */
	if (!write_new_stats()) { xchat_printf(ph, "%iERROR:: Could not save stats.", fs.colours[0]); }
//	xchat_commandf(ph, "say moo us 9");
	
	return;
}



/*
** Returns the total speed of all files XChat is uploading (in Kb/sec)
*/
int current_upload_speed(xchat_plugin *ph)
{
	int cs = 0;
	
	xchat_list *list;
	list = xchat_list_get(ph, "dcc");
	
	/* Go through the list of all active DCC sends and add the current speeds up */
	while(xchat_list_next(ph, list))
	{
		if ((xchat_list_int(ph, list, "status") == 1) && (xchat_list_int(ph, list, "type") == 0))
		{
			cs +=  xchat_list_int(ph, list, "cps");
		}
	}
	xchat_list_free(ph, list);
	
	/* Divide "cs" by 1024 to convert bytes/sec to Kilobytes/sec */
	cs = (cs / 1024);
	
	return cs;
}



/*
** Check if the user "nick" is already logged into the fserve
*/
int is_user_already_in(xchat_plugin *ph, char *nick)
{
	int i;
	char message[256];
	
	for (i = 0; i <= us[0].current_users; i++)
	{
		if (strcmp(us[i].users_name, nick) == 0)
		{
			/* Let them know that they're already logged in */
			snprintf(message, 256, "%iYou're already logged into this fserve.", fs.colours[4]);
			send_notice(ph, nick, message);
			return 1;
		}
	}
	
	return 0;
}



/*
** kick a certain user (word[3]) from the fserve
*/
void kick_user(xchat_plugin *ph, char *word[])
{
	int user = 0;
	
	/* Check if user is on the fserve */
	user = find_users_number(word[3]);
//	if (user == 30000)
//	{
//		/* Nope - not on the fserve */
//		xchat_printf(ph, "That user's not on your fserve.");
//		return;
//	}
	
	/* Make sure this is output to the correct window/tab/context */
	set_context(ph, us[user].users_name);
	
	/* Let the user know they're kicked */
	xchat_commandf(ph, "say %iYou have been kicked from this fserve. All your sends and/or queues will remain and continue as normal.", fs.colours[4]);
	
	/* Kill the DCC chat session */
	xchat_commandf(ph, "dcc close chat %s", us[user].users_name);
	if (fs.auto_close) { xchat_command(ph, "close"); }
	
	/* Remove them from the list of currently connected users */
	user_left_fserve(ph,us[user].users_name );
	
	return;
}



/*
** Bans a nick or host mask from the fserve_settings
*/
void ban_user(xchat_plugin *ph, char *word[])
{
	int i;
	char read_string[256];
	FILE *ban_list;
	
	/* Open the ban_list file - read/write */
	sprintf(gs.temp_path, "%sban_list", gs.tuxserve_path);
	if ((ban_list = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Read through the ban_list to see if this nick/mask is already banned */
		while (fgets(read_string, 256, ban_list))
		{
			remove_newline_char(read_string);
			if (strcmp(read_string, word[3]) == 0)
			{
				/* Yep, already exists */
				xchat_printf(ph, "That is already in the bans list.");
				fclose(ban_list);
				return;
			}
		}
		
		/* If it contains a "*" char, let the user know that TuxServe doesn't
		** yet support wildcard bans */
		for (i = 0; i < (strlen(word[3]) + 1); i++)
		{
			if (word[3][i] == '*')
			{
				xchat_printf(ph, "04Ban not added: TuxServe does not currently support wildcards (*) in ban masks. Please go bitch at Aypok (irc.odinnet.ca/#oldskool) until he adds it.");
				fclose(ban_list);
				return;
			}
		}
		
		/* Close the file and open in append mode */
		fclose(ban_list);
		if ((ban_list = fopen(gs.temp_path, "a")) != NULL)
		{
			/* If we get here, it must not already be in the bans list - so add it */
			sprintf(read_string, "%s\n", word[3]);
//			xchat_printf(ph, "ban=\"%s\"", read_string);
			fputs(read_string, ban_list);
			
			/* Report back */
			xchat_printf(ph, "04\"%s\" is now banned from your fserve.", word[3]);
			if (fs.inform_banned)
			{
				xchat_printf(ph, "If the user tries to connect to your fserve, they we will be notified of their ban. You can disable this by setting \"inform_banned\" off (\"/ts config set inform_banned 0\").");
			} else
			{
				xchat_printf(ph, "If the user tries to connect to your fserve, they we will be ignored. If you want them to be notified of their ban, set \"inform_banned\" on (\"/ts config set inform_banned 1\").");
			}
		} else
		{
			/* Can't open ban_file */
			xchat_printf(ph, "04ERROR:: Can't open the ban_list file.");
			return;
		}
	} else
	{
		/* Can't open ban_file */
		xchat_printf(ph, "04ERROR:: Can't open the ban_list file.");
		return;
	}
	
	fclose(ban_list);
	return;
}



/*
** Unbans a nick or host mask from the fserve_settings
*/
void unban_user(xchat_plugin *ph, char *word[])
{
	char read_string[256], second_string[256], *nick_read = NULL;
	FILE *ban_list, *temp_ban_list = NULL;
	
	/* Open the ban_list file - read only */
	sprintf(gs.temp_path, "%sban_list", gs.tuxserve_path);
	if ((ban_list = fopen(gs.temp_path, "r+")) != NULL)
	{
		/* Open a temp file */
//		if ((temp_ban_list = tmpfile()) == NULL)
		if (!(temp_ban_list = open_temp_file(temp_ban_list)))
		{
			xchat_printf(ph, "04ERROR:: Can't open the temp file.");
			fclose(ban_list);
			return;
		}
			
		/* Read through the ban_list and copy all, EXCEPT the nick/mask we want
		** to remove, to the temp file */
		while (fgets(read_string, 256, ban_list))
		{
			strcpy(second_string, read_string);
			nick_read = remove_newline_char(second_string);
			if (strcmp(nick_read, word[3]) != 0)
			{
				fputs(read_string, temp_ban_list);
			}
		}
		
		/* Close and clear the ban_list file, then reopen */
		fclose(ban_list);
		truncate(gs.temp_path, 0);
		
		/* Opening the file as writable */
		if ((ban_list = fopen(gs.temp_path, "w")) != NULL)
		{
			/* Copy the temp file back to the ban_list file */
			rewind(temp_ban_list);
			while (fgets(read_string, 256, temp_ban_list))
			{				
				fputs(read_string, ban_list);
			}
			
			/* Clean up a bit */
			fclose(temp_ban_list);
			fclose(ban_list);
			
			xchat_printf(ph, "\"%s\" is now unbanned from you fserve.", word[3]);
			
			return;
		} else
		{
			/* Can't open ban_file */
			xchat_printf(ph, "04ERROR:: Can't open the ban_list file. Letting in all users.");
			return;
		}
	} else
	{
		/* Can't open ban_file */
		xchat_printf(ph, "04ERROR:: Can't open the ban_list file. Letting in all users.");
		return;
	}
}



/*
** Checks if a user is banned from the fserve
*/
int is_user_banned(xchat_plugin *ph, char *nick)
{
	char *nick_address = NULL, message[256], nick_hostname[256], nick_check[256];
	FILE *ban_list;
	xchat_list *list;
	
	/* Find host mask of this user */
	list = xchat_list_get(ph, "users");
	if (list)
	{
		while(xchat_list_next(ph, list))
		{
			/* Have we found their nick in the list? */
			if (strcmp(xchat_list_str(ph, list, "nick"), nick) == 0)
			{
				snprintf(nick_hostname, 256, "%s", xchat_list_str(ph, list, "host"));
				snprintf(nick_check, 256, "%s", xchat_list_str(ph, list, "host"));
			}
		}
	} else
	{
		/* Can't get users list */
		xchat_printf(ph, "04ERROR:: Can't get users list from XChat to check the host of the user trying to access your fserve. Banning by host is ignored.");
	}
	
	/* Open the ban_list file read only */
	sprintf(gs.temp_path, "%sban_list", gs.tuxserve_path);
	if ((ban_list = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Read through the ban_list to see if this nick/mask is banned */
		while (fgets(message, 256, ban_list))
		{
			remove_newline_char(message);
			
			/* If it's got no "@" symbol in the ban string, host address - get
			** the user's host address to check against it */
			nick_address = get_host_address(nick_check);
//			xchat_printf(ph, "nr=\"%s\" nick=\"%s\" nh=\"%s\" na=\"%s\"", nick_read, nick, nick_hostname, nick_address);
			
			if ((strcmp(message, nick) == 0) || (strcmp(message, nick_hostname) == 0) || (strcmp(message, nick_address) == 0))
			{
				/* They be banified */
				if (fs.inform_banned)
				{
					/* Let the user know that they're banned */
					sprintf(message, "%iYou are banned from this fserve.", fs.colours[4]);
					send_notice(ph, nick, message);
				}
				
				fclose(ban_list);
				return 1;
			}
		}
	} else
	{
		/* Can't open ban_file */
		xchat_printf(ph, "04ERROR:: Can't open the ban_list file. Letting in all users.");
		return 0;
	}
	
	/* Not banned */
	fclose(ban_list);
	return 0;
}



void show_bans(xchat_plugin *ph)
{
	char ban[256];
	FILE *ban_file;
	
	/* Open the ban_list file for reading */
	sprintf(gs.temp_path, "%sban_list", gs.tuxserve_path);
	if((ban_file = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Header type thing */
		xchat_printf(ph, "%i---", fs.colours[0]);
		xchat_printf(ph, "%iCurrently banned:", fs.colours[2]);
		xchat_printf(ph, "");
		
		/* Go through the ban file, printing out each ban that's in there */
		while (fgets(ban, 256, ban_file))
		{
			remove_newline_char(ban);
			xchat_printf(ph, "\"%s\"", ban);
		}
		
		xchat_printf(ph, "%i---", fs.colours[0]);
		fclose(ban_file);
	} else
	{
		/* Can't open bans file */
		xchat_printf(ph, "ERROR:: Can't open the ban list file ");
	}
}
