/*
** TuxServe - XChat plugin.
** (C) 2004 Aypok <aypok@users.sourceforge.net>
**
** TuxServe is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** TuxServe is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with TuxServe; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**
** Last updated 2004-12-22
**
**
** Most of the code from this file is pulled from the "unstable" version of
** TuxServe and made to work with the old stable series. That's why some things
** seem to be done differently
**
*/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "xchat-plugin.h"
#include "config.h"
#include "dcc.h"
#include "display.h"
#include "extern.h"
#include "main.h"
#include "menus.h"
#include "sends.h"



/*******************************************************************************
**
** Reads through the queues file until it finds "queue_number". It copies all of
** queues file, except that queue, to a temp file, then copies the temp file
** back to the queues file
**
*******************************************************************************/
int remove_from_queue(int *queue_number)
{
	int i;
	char remove_this_queue[256], read_string[256];
	FILE *queues_file, *temp_queues_file = NULL;
	
	/* Open the queues file read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{		
		/* Find the queue line to remove */
		for (i = 0; i < *queue_number; i++)
		{
//			xchat_commandf(ph, "moo1 = %i\n", i);
			/* Read this line into remove_this_queue */
			fgets(remove_this_queue, 256, queues_file);
			
			/* Check if we hit the EOF - return 3 = no such queue*/
//			if (remove_this_queue[0] == 'EOF') { return 3; }
		}
		
		/* Now we know what to remove, go back to the start of the file and copy
		** all of it, EXCEPT that line, to a temp queue file */
		rewind(queues_file);
		
//		if ((temp_queues_file = tmpfile()) == NULL)
		if (!(temp_queues_file = open_temp_file(temp_queues_file)))
		{
//			xchat_commandf(ph, "ERROR:: Can't open temp queue file");
			return 0;
		}
		
		/* Copy all queues file (except that 1 queue) to the temp file */
		while (fgets(read_string, 256, queues_file))
		{
//			xchat_commandf(ph, "moo3 = rs: \"%s\" rtq: \"%s\"\n", read_string, remove_this_queue);
			if (strcmp(read_string, remove_this_queue) != 0)
			{
				fputs(read_string, temp_queues_file);
//				xchat_commandf(ph, "moo4\n");
			}
		}
		
		/* Close and clear the queues file, then reopen */
		fclose(queues_file);
		truncate(gs.temp_path, 0);
		
		/* Opening the file as writable */
		if ((queues_file = fopen(gs.temp_path, "w")) != NULL)
		{
			/* Copy the temp file back to the queues file */
			rewind(temp_queues_file);
			while (fgets(read_string, 256, temp_queues_file))
			{
//				xchat_commandf(ph, "moo5 = rs: \"%s\"\n", read_string);
				
				fputs(read_string, queues_file);
			}
		
			/* Clean up a bit */
			fclose(temp_queues_file);
			fclose(queues_file);
			
			fs.queue--;
		}
	
	}
	
	return 1;
}



/*******************************************************************************
**
** Removes all queues for a certain nick. It reads through the queues file until
** we find the nick, then pass the queue number to "remove_from_queue"
**
*******************************************************************************/
int remove_named_queues(char *sender)
{
	int i = 0;
	char read_string[256], check_string[64];
	FILE *queues_file;
	
	/* Open the queues file - read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Their non-prefixed nick */
		strcpy(check_string, sender);
		remove_mode_prefix(check_string);
		
		/* Read through the queues file looking for "data1" (their nick) */
		while (fgets(read_string, 256, queues_file))
		{			
			/* If it's the nick, or we're to remove everything - pass the queue
			** number to "remove_from_queue" */
			get_setting_name(read_string);
			
			if ((strcmp(read_string, sender) == 0) || (strcmp(read_string, check_string) == 0) || (strcmp(sender, "@ts_everything@") == 0))
			{
				/* Now that we have the number, we can call the function to
				** remove it. REMEMBER: the number to remove is the actual
				** queue number + 1! */
				i++;
//				xchat_commandf(ph, "moo - \"%s\"==\"%s\" = %i\n", data1, nick_string, i);
				if (!remove_from_queue(&i))
				{
					/* Failed */
					return 0;
				}
				
				/* Now that a queue has been removed from the queue, it'll be 1
				** queue shorter. So, the next queue isn't "i + 1", it's "i" - 
				** the queue in "i" has been removed, so the queue in "i + 1" is
				** now "i". Why do we -2? We just added 1 so we could pass it to
				** get removed - we don't need that either */
				i = (i - 2);
			}
			
			/* "i" counts the queues, so we know which queue to remove */
			i++;
		}
		
		/* All done */
		fclose(queues_file);
		return 1;
	}
	
	/* If we get here, then something failed */
	return 0;
}



/*******************************************************************************
**
** Read the queues file and read the first line containing data. Then separate
** the name of the person who queued it and the file they queued, then send them
** back to the calling function
**
*******************************************************************************/
int get_next_queue(xchat_plugin *ph)
{
	int i = 0, flag_queues = 0;
	char read_string[256], save_string[256];
	FILE *queues_file;
	
	/* Open the queues file - read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{
		while (fgets(read_string, 256, queues_file))
		{
			/* If it's not blank, it's our first queued file */
			if (read_string[0] != '\n')
			{
				++i;
				flag_queues = 1;
				
				/* Now, find who queued the file */
				strcpy(save_string, read_string);
				get_setting_name(save_string);
				remove_mode_prefix(save_string);
				
				/* Check if they have any free send slots */
				if (check_user_sends(save_string))
				{
					/* Yep, they do */
					fclose(queues_file);
					
					/* Add the newline char to the end of the first string - the
					** second string will already have a newline char */
					get_setting_value(read_string);
//					sprintf(save_string, "%s\n", read_string);
					
					/* Now remove the file from the queues file. We don't care
					** about the return value of "remove_from_queue" yet */
					remove_from_queue(&i);
					
					/* Remove the newlines from the strings */
//					remove_newline_char(save_string);
					remove_newline_char(read_string);
					
					/* Send file */
					send_file(ph, read_string, save_string, "-");
					
					return 1;
				}
			}
		}
		/* Looks like no-one in the queue has free send slots - wait until the
		** next send ends and try again */
		fclose(queues_file);
	}
	return 0;
}



/*******************************************************************************
**
** Add a new file to the end of the queue
**
*******************************************************************************/
int add_queue(char *data1, char *data2)
{
	int flag = 0;
	char read_string[256], spare_string[256];
	FILE *queues_file, *temp_queues_file = NULL;
	
	/* Create the string to write to the file */
	sprintf(read_string, "%s}%s\n", data1, data2);
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	
	/* Does this persons name start with a prefix (+, %, @, &, ~) - if so, give
	** them priority queuing. Go to the start of the queues file and read each
	** queue - when it comes across a non-prefixed user it will insert this
	** persons queue there. This is all done via a temp file, as with a lot of
	** things in TuxServe. If it's not a prefixed user, add them to the end of
	** the queue */
	if ((read_string[0] == '+') || (read_string[0] == '%') || (read_string[0] == '@') || (read_string[0] == '&') || (read_string[0] == '~'))
	{
		/* Yep, this dude(ette) has priority - make way */
		if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
		{		
			/* Open a temp file */			
//			if ((temp_queues_file = tmpfile()) == NULL) { return 0; }
			if (!(temp_queues_file = open_temp_file(temp_queues_file))) { return 0; }
			
			/* Read all queues one by one and check if the user's name starts
			** with a prefix. If it does, copy it to the temp file. If not, and
			** "flag" is not set, then add the new queue here, then copy the
			** rest of the queues */
			while (fgets(spare_string, 256, queues_file))
			{
				if (((spare_string[0] != '+') && (spare_string[0] != '%') && (spare_string[0] != '@') && (spare_string[0] != '&') && (spare_string[0] != '~')) && (!flag))
				{
					/* This is at the end of all priority queues, and the start
					** of normal queues. Write this new queue here */
					fputs(read_string, temp_queues_file);
					/* Set "flag" so we only add the new queue once */
					flag = 1;
				}				
				
				/* Add the rest of the queues back */
				fputs(spare_string, temp_queues_file);
			}
			
			/* Close and clear the queues file, then reopen */
			fclose(queues_file);
			truncate(gs.temp_path, 0);
			
			/* Open this file as writable */
			if ((queues_file = fopen(gs.temp_path, "w")) != NULL)
			{
				/* Copy the temp file back to the queues file */
				rewind(temp_queues_file);
				while (fgets(read_string, 256, temp_queues_file))
				{					
					fputs(read_string, queues_file);
				}
			
				/* Clean up a bit */
				fclose(temp_queues_file);
				fclose(queues_file);
				
				/* All done - lets party */
				return 1;
			} else
			{
				/* Can't open file */
				return 0;
			}
		} else
		{
			/* Can't open file */
			return 0;
		}
	} else
	{
		/* Open the queues file to append the new data to it */
		if ((queues_file = fopen(gs.temp_path, "a")) != NULL)
		{
			/* Create the string to write to the file */
			sprintf(read_string, "%s}%s\n", data1, data2);
			/* Write it to the file, then close it */
			fputs(read_string, queues_file);
			fclose(queues_file);
			
			fs.queue++;
			
			return 1;
		} else
		{
			/* Can't open file */
			return 0;
		}
	}
}



/*******************************************************************************
**
** Check if the user has already queued that file
**
*******************************************************************************/
void does_queue_exist(xchat_plugin *ph, char *user_name, char *file_name)
{
	int user;
	char check_string[256], check_pri_string[256], read_string[256];
	FILE *queues_file;
	
	/* Find the number of this user */
	user = find_users_number(user_name);
	
	/* Open the queues file - read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{		
		/* Create the strings to check against - the "_pri_" is for checking if
		** this person has any priority queues. We need to check their nick with
		** and without their prefix - otherwise they could get extra queues */
		snprintf(check_pri_string, 256, "%s}%s\n", user_name, file_name);
		
		/* Their non-prefixed nick */
		strcpy(read_string, user_name);
		remove_mode_prefix(read_string);
		snprintf(check_string, 256, "%s}%s\n", read_string, file_name);
		
		/* Read through the whole file, checking each line for a match */
		while (fgets(read_string, 256, queues_file))
		{
			
			if ((strcmp(read_string, check_string) == 0) || (strcmp(read_string, check_pri_string) == 0))
			{
				/* Found it - this queue already exists */
				fclose(queues_file);
				xchat_commandf(ph, "say %iThat queue already exists.", fs.colours[0]);
				return;
			}
		}
		fclose(queues_file);
		
		/* Queue doesn't exist - so add it to the queue */
		if (add_queue(user_name, file_name))
		{
			us[user].queued_flag = 1;
			remove_path(file_name);
			xchat_commandf(ph, "say %iFile \"%i%s%i\" added to queue in position%i %i.", fs.colours[4], fs.colours[3], file_name, fs.colours[4], fs.colours[2], fs.queue);
		}
		return;
	} else
	{
		/* Brokened */
		xchat_printf(ph, "%iERROR:: Can't open queues file", fs.colours[0]);
		return;
	}
	
	return;
}



/*******************************************************************************
**
** Checks if there are any queues free for a certain nick (sender). If "@ts_all@"
** is sent, then it will just count all queues. If they have no free queue slots,
** then "0" is returned - otherwise the number of queues they have is returned
**
*******************************************************************************/
int query_free_queues(xchat_plugin *ph, char *sender)
{
	int queues_count = 0;
	char read_string[256], check_string[64];
	FILE *queues_file;
	
	/* Open the queues file - read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{		
		/* Their non-prefixed nick */
		strcpy(check_string, sender);
		remove_mode_prefix(check_string);
		
		/* Read through the whole file, counting each line */
		while (fgets(read_string, 256, queues_file))
		{
			/* If this is the nick of who we want to check for, increase
			** their queues count. If "sender" = "@ts_all@" then return the
			** number of queued files */
			get_setting_name(read_string);
			
//			xchat_commandf(ph, "say rs=\"%s\" sender=\"%s\" cs=\"%s\"", read_string, sender, check_string);
			if ((strcmp(read_string, sender) == 0) || (strcmp(read_string, check_string) == 0) || ((strstr(sender, "@ts_all@")) && (read_string[0] != '\n')))
			{
				queues_count++;
			}
		}

		fclose(queues_file);
	} else
	{
		/* Brokened */
		xchat_printf(ph, "%iERROR:: Can't open queues file", fs.colours[0]);
		return 0;
	}
	
	if (strcmp(sender, "@ts_all@") == 0) { return queues_count; }
	
	/* If "fs.user_queues" is greater than "queues_count", then there are free
	** queue slots */
//	if (fs.user_queues > queues_count)
//	{
		/* Free queues */
//		xchat_commandf(ph, "say FREE :: qa=%i > qc=%i", fs.user_queues, queues_count);
//		xchat_printf(ph, "%s", sender);
/*		if (strstr(sender, "@ts_all@"))
		{
			return queues_count;
		} else
		{
			return 1;
		}
	}*/
//	xchat_commandf(ph, "say FULL :: qa=%i > qc=%i", fs.user_queues, queues_count);

	return queues_count;
//	return 0;
}



/*******************************************************************************
**
** Responds to "queues", when typed in the DCC fserve chat session, with a list
** of all currently queued files, and who queued them. If "show_type" == "0"
** then it was called from the /ts command and must be printed to the screen. If
** "show_type" is "1", then it was called by a user from the fserve DCC chat
** session. If "show_type" == "2", then it's from a user using the "/ctcp <your
** nick> my_queues" trigger
**
*******************************************************************************/
void show_queues(xchat_plugin *ph, char *sender, int *show_type)
{
	int queue_number = 1, i = 0;
	long unsigned int size = 0;
	char readstring[256], tempstring[256], size_string[3], current_file[512];
	struct stat buf;
	FILE *queuesfile;
		
	/* If it's called by a user and not the person running TuxServe: */
//	if (strcmp(sender, "forinternalusebytuxserve") != 0)
	if (*show_type == 1)
	{
		set_context(ph, sender);
		xchat_commandf(ph, "say %i---", fs.colours[0]);
		xchat_commandf(ph, "say %iCurrent queues:%i %i/%i", fs.colours[2], fs.colours[4], fs.queue, fs.max_queues);
		xchat_commandf(ph, "say %i", fs.colours[0]);
	}
	
	/* Open the queues file for reading */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if((queuesfile = fopen(gs.temp_path, "r")) != NULL)
	{
		while (fgets(readstring, 256, queuesfile))
		{
			strcpy(tempstring, readstring);
			get_setting_name(tempstring);
			get_setting_value(readstring);
			remove_newline_char(readstring);
			
			/* Check it's not an empty line */
			if ((strcmp(tempstring, "") != 0) && (strcmp(readstring, "") != 0) && (strcmp(tempstring, " ") != 0) && (strcmp(readstring, " ") != 0) && (tempstring[0] != '\n') && (readstring[0] != '\n'))
			{
				/* Find the size of the queued file */
				sprintf(current_file, "%s", readstring);
//				xchat_printf(ph, "\"%s\" -> \"%s\"", current_file, queues_file);
				if (stat(current_file, &buf) != 0)
				{
					sprintf(size_string, "ERROR");
				} else
				{
					/* For displaying the size in bytes, kb, mb, gb, etc */
					size = buf.st_size;
					
					/* If the files size is less than 1kb */
					if (size < 1024) { sprintf(size_string, "b"); }
					
					/* If the file is between 1kb and 1mb */
					if ((size < 1048576) && (size > 1023)) { size = (size / 1024); sprintf(size_string, "kb"); }
					
					/* If the file is between 1mb and 1gb */
					if ((size < 1073741824) && (size > 1048575)) { size = (size / 1048576); sprintf(size_string, "mb"); }
					
					/* If it's over 1gb */
					if (size > 1073741823) { size = (size / 1073741824); sprintf(size_string, "gb"); }
				}
				
				/* Display the queues
				** To the person running TuxServe only */
//				if (strcmp(sender, "forinternalusebytuxserve") == 0)
				if (*show_type == 0)
				{
					xchat_printf(ph, "%iQueue %i:%i %s %i(%lu %s)%i queued by%i %s", fs.colours[2], queue_number, fs.colours[4], readstring, fs.colours[3], size, size_string, fs.colours[1], fs.colours[4], tempstring);
				}
				
				/* Say in DCC chat window */
				if (*show_type == 1)
				{
					/* Make sure we only show the filename and not the full path */
					remove_path(readstring);
					
					xchat_commandf(ph, "say %iQueue %i:%i %s %i(%lu %s)%i queued by%i %s", fs.colours[2], queue_number, fs.colours[4], readstring, fs.colours[3], size, size_string, fs.colours[1], fs.colours[4], tempstring);
				}
				
				/* Send via a notice */
				if (*show_type == 2)
				{
					if (strcmp(sender, tempstring) == 0)
					{
						/* Make sure we only show the filename and not the full path */
						remove_path(readstring);
						
						sprintf(tempstring, "%iQueue %i:%i %s %i(%lu %s)", fs.colours[2], queue_number, fs.colours[4], readstring, fs.colours[3], size, size_string);
						send_notice(ph, sender, tempstring);
						i++;
					}
				}
					
				queue_number++;
				
				/* Make sure it doesn't show too many things */
				if (queue_number > fs.queue)
				{
					/* Add the "footer" to the queues list */
					if (*show_type == 1) { xchat_commandf(ph, "say %i---", fs.colours[0]); }
					if ((*show_type == 2) && (!i)) { sprintf(tempstring, "%iYou have no files queued.", fs.colours[2]);	send_notice(ph, sender, tempstring); }
						
					fclose(queuesfile);
					return;
				}
			}
		}
	}	
	fclose(queuesfile);
	
	return;
}



/*******************************************************************************
**
** Returns the total size of all currently queued files. It's a char rather than
** an int, so we can add "b", "kb", "mb", or "gb" on the end of it. Saves having
** extra code in other places
**
*******************************************************************************/
char *size_of_queues(xchat_plugin *ph, char *stringy)
{
	int cs = 0;
	char readstring[256];
	struct stat buf;
	FILE *queuesfile;
	
	/* Open the queues file for reading */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if((queuesfile = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Read through each line in the queues file */
		while (fgets(readstring, 256, queuesfile))
		{
			get_setting_value(readstring);
			
			/* Get the file size of the queued file (in bytes) */
			if (stat(readstring, &buf)) { cs += buf.st_size; }
		}
	}
	
	/* If the files size is less than 1kb */
	if (cs < 1024) { sprintf(stringy, "%ib", cs); }
	
	/* If the file is between 1kb and 1mb */
	if ((cs < 1048576) && (cs > 1023)) { cs = (cs / 1024); sprintf(stringy, "%ikb", cs); }
	
	/* If the file is between 1mb and 1gb */
	if ((cs < 1073741824) && (cs > 1048575)) { cs = (cs / 1048576); sprintf(stringy, "%imb", cs); }
	
	/* If it's over 1gb */
	if (cs > 1073741823) { cs = (cs / 1073741824); sprintf(stringy, "%igb", cs); }
	
	return stringy;
}



/*******************************************************************************
**
** Checks the nick of whoever queued "queue_check" against "nick_check". If they
** match, then return 1 - else, return 0
**
*******************************************************************************/
int is_this_their_queue(xchat_plugin *ph, int *queue_check, char *nick_check)
{
	int i;
	char read_string[256], check_string[64];
	FILE *queues_file;
	
	/* Open the queues file read only */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	if ((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Their non-prefixed nick */
		strcpy(check_string, nick_check);
		remove_mode_prefix(check_string);
		
		i = 0;
		/* Read through each line in the queues file */
		while (fgets(read_string, 256, queues_file))
		{	
			if (*queue_check == (++i)) { break; }
		}
		fclose(queues_file);
		
		/* Check if we've broken the loop because we found the queue number, of
		** because we've hit the EOF */
		if (*queue_check != i) { return 0; }
			
		/* Check the nicks */
		get_setting_name(read_string);
//		xchat_commandf(ph, "say rs=\"%s\" nc=\"%s\" qc=\"%i\"", read_string, nick_check, *queue_check);
		if ((strcmp(read_string, nick_check) == 0) || (strcmp(read_string, check_string)))
		{
			/* This is their queue */
			return 1;
		} else
		{
			/* Nope - someone else's queue */
			return 0;
		}
	}
	
	/* Error reading file */
	return 0;
}



/*******************************************************************************
**
** Removes the specified file from the queues file
**
*******************************************************************************/
void remove_queue(xchat_plugin *ph, char *sends_file, char *sends_name)
{
	int i;
	char complete_string[256], complete_pri_string[256], read_string[256];
	FILE *queues_file, *tempfile = NULL;
	
	/* Set the path to the queues file */
	sprintf(gs.temp_path, "%squeues", gs.tuxserve_path);
	
	/* Open a temp file for writing to */
//	if((tempfile = tmpfile()) == NULL) { xchat_printf(ph, "%iERROR:: Can't opening temp file.", fs.colours[0]); return; }
	if (!(tempfile = open_temp_file(tempfile))) { xchat_printf(ph, "%iERROR:: Can't opening temp file.", fs.colours[0]); return; }
	
	/* Build the string to search for */
	sprintf(complete_pri_string, "%s}%s\n", sends_name, sends_file);

	/* Build the non-prefixed string to search for */
	strcpy(read_string, sends_name);
	remove_mode_prefix(read_string);
	snprintf(complete_string, 256, "%s}%s\n", read_string, sends_file);
	
	/* Open the queues file for writing */
	if((queues_file = fopen(gs.temp_path, "r")) != NULL)
	{
		/* Search through the queues file and copy all of it, except the line
		** contained in "complete_string", to the temp file */
		fgets(read_string, 256, queues_file);
		for(i = 0; i < fs.queue; i++)
		{
			fgets(read_string, 256, queues_file);
			/* If what's read in is NOT the same as "complete_string": */
			if ((strcmp(read_string, complete_string) != 0) || (strcmp(read_string, complete_pri_string) != 0))
			{
				/* Write the line to the temp file */
				fputs(read_string, tempfile);
			}
		}	
	}
	
	/* Close the queues file, empty it, then reopen it */
	fclose(queues_file);
	rewind(tempfile);
		
	truncate(gs.temp_path, 0);
	if((queues_file = fopen(gs.temp_path, "w")) == NULL)
	{
		xchat_printf(ph, "%iERROR:: Can't open the sends file.", fs.colours[0]);
		return;
	}
	
	/* Now copy all of the temp file into the queues file */
	while (fgets(read_string, 256, tempfile))
	{
		fputs(read_string, queues_file);
	}
	
	/* Close all the files and return */
	fflush(queues_file);
	fclose(queues_file);
	fclose(tempfile);
	
	return;
}
